@@ -1,5 +1,5 @@
 
-/* Copyright (c) 2016, 2017, 2018, 2019, 2020 Thorsten Kukuk
+/* Copyright (c) 2016, 2017, 2018, 2019, 2020, 2023 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.com>
 
    This program is free software; you can redistribute it and/or modify
@@ -41,6 +41,9 @@
 #define PROPERTIES_METHOD_GET    "Get"
 #define PROPERTIES_METHOD_SET    "Set"
 
+static int verbose_flag = 0;
+static int log_type = LOG_DEBUG;
+
 static RM_CTX *ctx;
 static pthread_mutex_t mutex_ctx = PTHREAD_MUTEX_INITIALIZER;
 
@@ -88,7 +91,9 @@ print_help (void)
 {
   fputs (_("rebootmgrd - reboot following a specified strategy\n\n"), stdout);
 
-  fputs (_("  -d,--debug   Log debug output\n"),
+  fputs (_("  -d,--debug     Debug mode, no reboot done\n"),
+         stdout);
+  fputs (_("  -v,--verbose   Verbose logging\n"),
          stdout);
   fputs (_("  -?, --help     Give this help list\n"), stdout);
   fputs (_("      --version  Print program version\n"), stdout);
@@ -104,7 +109,7 @@ print_error (void)
 }
 
 static void
-reboot_now (void)
+reboot_now (int soft_reboot)
 {
   pthread_mutex_lock (&mutex_ctx);
   if (ctx->temp_off)
@@ -118,7 +123,10 @@ reboot_now (void)
       if (!debug_flag)
 	{
 	  pthread_mutex_unlock (&mutex_ctx);
-	  log_msg (LOG_INFO, "rebootmgr: reboot triggered now!");
+	  if (soft_reboot)
+	    log_msg (LOG_INFO, "rebootmgr: soft-reboot triggered now!");
+	  else
+	    log_msg (LOG_INFO, "rebootmgr: reboot triggered now!");
 	  pid_t pid = fork();
 
 	  if (pid < 0)
@@ -127,17 +135,34 @@ reboot_now (void)
 	    }
 	  else if (pid == 0)
 	    {
-	      if (execl ("/usr/bin/systemctl", "systemctl", "reboot",
-			 NULL) == -1)
+	      if (soft_reboot)
 		{
-		  log_msg (LOG_ERR, "Calling /usr/bin/systemctl failed: %m");
-		  exit (1);
+		  if (execl ("/usr/bin/systemctl", "systemctl", "soft-reboot",
+			     NULL) == -1)
+		    {
+		      log_msg (LOG_ERR, "Calling /usr/bin/systemctl soft-reboot failed: %m");
+		      exit (1);
+		    }
+		}
+	      else
+		{
+		  if (execl ("/usr/bin/systemctl", "systemctl", "reboot",
+			     NULL) == -1)
+		    {
+		      log_msg (LOG_ERR, "Calling /usr/bin/systemctl reboot failed: %m");
+		      exit (1);
+		    }
 		}
 	      exit (0);
 	    }
 	}
       else
-	log_msg (LOG_DEBUG, "systemctl reboot called!");
+	{
+	  if (soft_reboot)
+	    log_msg (LOG_DEBUG, "systemctl soft-reboot called!");
+	  else
+	    log_msg (LOG_DEBUG, "systemctl reboot called!");
+	}
 
       ctx->reboot_status = RM_REBOOTSTATUS_NOT_REQUESTED;
     }
@@ -147,25 +172,25 @@ reboot_now (void)
 /* Check which reboot method and forward to that function */
 /* Called by timer_create as new thread */
 static void
-reboot_timer (sigval_t RM_UNUSED(user_data))
+reboot_timer (sigval_t soft_reboot)
 {
   if (debug_flag)
     log_msg (LOG_DEBUG, "reboot_timer called");
 
-    reboot_now ();
+    reboot_now (soft_reboot.sival_int);
 }
 
 /* Create a new timer thread, which calls '_function' after
    specified seconds */
 static timer_t
-create_timer (time_t seconds, void (*_function) (sigval_t))
+create_timer (time_t seconds, int soft_reboot, void (*_function) (sigval_t))
 {
   timer_t timer_id;
 
   /* Create timer */
   struct sigevent se;
   se.sigev_notify = SIGEV_THREAD;
-  se.sigev_value.sival_ptr = NULL;
+  se.sigev_value.sival_int = soft_reboot;
   se.sigev_notify_function = _function;
   se.sigev_notify_attributes = NULL;
   if (timer_create(CLOCK_REALTIME, &se, &timer_id) == -1)
@@ -191,7 +216,7 @@ create_timer (time_t seconds, void (*_function) (sigval_t))
 }
 
 static void
-initialize_timer (void)
+initialize_timer (int soft_reboot)
 {
   int r;
   usec_t curr = now(CLOCK_REALTIME);
@@ -215,7 +240,7 @@ initialize_timer (void)
     {
       /* we are inside the maintenance window, reboot */
       pthread_mutex_unlock (&mutex_ctx);
-      reboot_timer ((sigval_t) 0);
+      reboot_timer ((sigval_t) soft_reboot);
       return;
     }
 
@@ -233,12 +258,12 @@ initialize_timer (void)
      everything at the beginning of the maintenance window */
   next = next + ((usec_t)rand() * USEC_PER_SEC) % duration;
 
-  if (debug_flag)
+  if (debug_flag || verbose_flag)
     {
       char buf[FORMAT_TIMESTAMP_MAX];
       int64_t in_secs = (next - curr) / USEC_PER_SEC;
 
-      log_msg (LOG_DEBUG,
+      log_msg (log_type,
 	       "Reboot in %i seconds at %s", in_secs,
 	       format_timestamp(buf, sizeof(buf), next));
     }
@@ -248,7 +273,7 @@ initialize_timer (void)
     timer_delete(ctx->reboot_timer_id);
 
   ctx->reboot_timer_id =
-    create_timer ((next - curr) / USEC_PER_SEC, reboot_timer);
+    create_timer ((next - curr) / USEC_PER_SEC, soft_reboot, reboot_timer);
 
   pthread_mutex_unlock (&mutex_ctx);
 }
@@ -262,47 +287,52 @@ do_reboot (void)
 
   ctx->reboot_status = RM_REBOOTSTATUS_REQUESTED;
 
-  if (ctx->reboot_order == RM_REBOOTORDER_FORCED)
+  if (ctx->reboot_order == RM_REBOOTORDER_FORCED ||
+      ctx->reboot_order == RM_REBOOTORDER_SOFT_FORCED)
     {
-      if (debug_flag)
-	log_msg (LOG_DEBUG, "Forced reboot requested");
+      if (debug_flag || verbose_flag)
+	log_msg (log_type, "Forced reboot requested");
       pthread_mutex_unlock (&mutex_ctx);
-      reboot_now ();
+      reboot_now (ctx->reboot_order == RM_REBOOTORDER_SOFT);
       return;
     }
 
+  int soft_reboot = ctx->reboot_order == RM_REBOOTORDER_SOFT || ctx->reboot_order == RM_REBOOTORDER_SOFT_FAST || ctx->reboot_order == RM_REBOOTORDER_SOFT_FORCED;
+
   switch (ctx->reboot_strategy)
     {
     case RM_REBOOTSTRATEGY_BEST_EFFORT:
       if (ctx->maint_window_start != NULL &&
-	  ctx->reboot_order != RM_REBOOTORDER_FAST)
+	  ctx->reboot_order != RM_REBOOTORDER_FAST &&
+	  ctx->reboot_order != RM_REBOOTORDER_SOFT_FAST)
 	{
 	  pthread_mutex_unlock (&mutex_ctx);
-	  initialize_timer();
+	  initialize_timer(soft_reboot);
 	  return;
 	}
       else
 	{
 	  pthread_mutex_unlock (&mutex_ctx);
-	  reboot_now ();
+	  reboot_now (soft_reboot);
 	  return;
 	}
       break;
     case RM_REBOOTSTRATEGY_INSTANTLY:
       pthread_mutex_unlock (&mutex_ctx);
-      reboot_now ();
+      reboot_now (soft_reboot);
       return;
       break;
     case RM_REBOOTSTRATEGY_MAINT_WINDOW:
       if (ctx->reboot_order == RM_REBOOTORDER_FAST ||
+	  ctx->reboot_order == RM_REBOOTORDER_SOFT_FAST ||
 	  ctx->maint_window_start == NULL)
 	{
 	  pthread_mutex_unlock (&mutex_ctx);
-	  reboot_now ();
+	  reboot_now (soft_reboot);
 	  return;
 	}
       pthread_mutex_unlock (&mutex_ctx);
-      initialize_timer ();
+      initialize_timer (soft_reboot);
       return;
       break;
     case RM_REBOOTSTRATEGY_OFF:
@@ -340,18 +370,33 @@ handle_native_iface (DBusMessage *message)
 	{
 	  if (order == RM_REBOOTORDER_STANDARD)
 	    {
-	      if (debug_flag)
-		log_msg (LOG_DEBUG, "Reboot at next possible time");
+	      if (debug_flag || verbose_flag)
+		log_msg (log_type, "Reboot at next possible time requested");
 	    }
 	  else if (order == RM_REBOOTORDER_FAST)
 	    {
-	      if (debug_flag)
-		log_msg (LOG_DEBUG, "Reboot as fast as possible");
+	      if (debug_flag || verbose_flag)
+		log_msg (log_type, "Fast reboot requested");
 	    }
 	  else if (order == RM_REBOOTORDER_FORCED)
 	    {
-	      if (debug_flag)
-		log_msg (LOG_DEBUG, "Reboot now");
+	      if (debug_flag || verbose_flag)
+		log_msg (log_type, "Immediate reboot requested");
+	    }
+	  else if (order == RM_REBOOTORDER_SOFT)
+	    {
+	      if (debug_flag || verbose_flag)
+		log_msg (log_type, "Soft reboot at next possible time requested");
+	    }
+	  else if (order == RM_REBOOTORDER_SOFT_FAST)
+	    {
+	      if (debug_flag || verbose_flag)
+		log_msg (log_type, "Fast soft reboot requested");
+	    }
+	  else if (order == RM_REBOOTORDER_SOFT_FORCED)
+	    {
+	      if (debug_flag || verbose_flag)
+		log_msg (log_type, "Immediate soft reboot requested");
 	    }
 	  else
 	    {
@@ -610,7 +655,7 @@ dbus_filter (DBusConnection *connection, DBusMessage *message,
       log_msg (LOG_INFO, "Lost connection to D-Bus");
       dbus_connection_unref (connection);
       connection = NULL;
-      create_timer (1, dbus_reconnect);
+      create_timer (1, 0, dbus_reconnect);
       handled = DBUS_HANDLER_RESULT_HANDLED;
     }
 
@@ -766,7 +811,8 @@ main (int argc, char **argv)
       static struct option long_options[] =
 	{
 	  {"debug", no_argument, NULL, 'd'},
-	  {"version", no_argument, NULL, 'v'},
+	  {"verbose", no_argument, NULL, 'v'},
+	  {"version", no_argument, NULL, '\255'},
 	  {"usage", no_argument, NULL, '?'},
 	  {"help", no_argument, NULL, 'h'},
 	  {NULL, 0, NULL, '\0'}
@@ -780,12 +826,17 @@ main (int argc, char **argv)
         {
         case 'd':
 	  debug_flag = 1;
+	  log_type = LOG_DEBUG;
 	  break;
 	case '?':
 	case 'h':
           print_help ();
           return 0;
 	case 'v':
+	  verbose_flag = 1;
+	  log_type = LOG_INFO;
+	  break;
+	case '\255':
 	  fprintf (stdout, "rebootmgrd (%s) %s\n", PACKAGE, VERSION);
           return 0;
         default:
